package com.izforge.izpack.matcher;

import org.hamcrest.Description;
import org.hamcrest.Matcher;
import org.hamcrest.MatcherAssert;
import org.hamcrest.TypeSafeMatcher;
import org.hamcrest.collection.IsCollectionContaining;
import org.hamcrest.core.Is;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;

/**
 * Matcher for zip files
 *
 * @author Anthonin Bonnefoy
 */
public class ZipMatcher extends TypeSafeMatcher<File>
{
    private Matcher<Iterable<String>> listMatcher;

    ZipMatcher(Matcher<Iterable<String>> listMatcher)
    {
        this.listMatcher = listMatcher;
    }

    @Override
    public boolean matchesSafely(File file)
    {
        try
        {
            List<String> fileList = getFileNameListFromZip(file);
            // MatcherAssert.assertThat(fileList, listMatcher); // This prevents the use of IsNot. TODO
            return listMatcher.matches(fileList);
        }
        catch (IOException e)
        {
            throw new AssertionError(e);
        }
    }


    public static List<String> getFileNameListFromZip(File file)
            throws IOException
    {
        List<String> fileList = new ArrayList<String>();
        FileInputStream fis = new FileInputStream(file);
        ZipInputStream zis = new ZipInputStream(fis);
        ZipEntry ze;
        while ((ze = zis.getNextEntry()) != null)
        {
            fileList.add(ze.getName());
            zis.closeEntry();
        }
        zis.close();
        return fileList;
    }

    public void describeTo(Description description)
    {
        description.appendText("Excepting collection containing ").appendValue(listMatcher);
    }

    public static ZipMatcher isZipContainingFile(String fileName)
    {
        return new ZipMatcher(IsCollectionContaining.hasItem(Is.is(fileName)));
    }

    public static ZipMatcher isZipContainingFiles(String... fileName)
    {
        return new ZipMatcher(IsCollectionContaining.hasItems(fileName));
    }

    public static ZipMatcher isZipMatching(Matcher<Iterable<String>> matcher)
    {
        return new ZipMatcher(matcher);
    }


}
